:- include( 'processes' ).
:- include( 'muc' ).

processdef( lamp, [nolight] * ( [on=false, lamp] + [on=true, lampon] ) ).
processdef( lampon, [light] * ( [on=true, lampon] + [on=false, lamp] ) ).

processdef( car, [send(wait), car_wait(5)] ).
processdef( car_wait(N), [go=true,car_go] + [tick,car_wait(N1)], N1 is N-1).
processdef( car_wait(0), car ).
processdef( car_go, [send(enter), tick(4),send(exit), car ] ).

processdef( lightctr, [llight:=false,rlight:=false,
    [update(lwait1),lgreen] + [update(lwait2),lgreen] +
    [update(rwait1),rgreen] + [update(rwait2),rgreen] ] ).
processdef( lgreen, [llight:=true,tick(6),lightleft] ).
processdef( rgreen, [rlight:=true,tick(6),lightright] ).
processdef( lightleft, [send(getempty),
    [empty=true,llight:=false,lightctr]+[empty=false,lightleft]] ).
processdef( lightright, [send(getempty),
    [empty=true,rlight:=false,lightctr]+[empty=false,lightright]] ).

processdef( bridge, bridge(0) ).
processdef( bridgel(0), bridge(0) ).
processdef( bridger(0), bridge(0) ).

processdef( bridge(0), 
    [update(lin1),bridgel(1)] + [update(lin2),bridgel(1)] +
    [update(rin1),bridger(1)] + [update(rin2),bridger(1)] +
    [update(request),empty:=true,bridge(0)] ).

processdef( bridgel(N), [fromleft] * (
    [update(lin1),bridgel(NP)] + [update(lout1),bridgel(NM)] + 
    [update(lin2),bridgel(NP)] + [update(lout2),bridgel(NM)] + 
    [update(request), [fromleft] * [empty:=false,bridgel(N)]] ),
    (NP is N + 1, NM is N - 1) ).

processdef( bridger(N), [fromright] * (
    [update(rin1),bridger(NP)] + [update(rout1),bridger(NM)] + 
    [update(rin2),bridger(NP)] + [update(rout2),bridger(NM)] + 
    [update(request), [fromright] * [empty:=false,bridger(N)]] ),
    (NP is N + 1, NM is N - 1) ).

net( oneway, 
    [lcar1:car, lcar2:car, rcar1:car, rcar2:car,
     b:bridge, c:lightctr, llight:lamp, rlight:lamp],
    [lcar1^enter/b^lin1, lcar1^leave/b^lout1, lcar1^wait/c^lwait1, 
     lcar2^enter/b^lin2, lcar2^leave/b^lout2, lcar2^wait/c^lwait2, 
     rcar1^enter/b^rin1, rcar1^leave/b^rout1, rcar1^wait/c^rwait1, 
     rcar2^enter/b^rin2, rcar2^leave/b^rout2, rcar2^wait/c^rwait2, 
     b^empty/c^empty, c^getempty/b^request,
     c^llight/llight^on, c^llight/lcar1^go, c^llight/lcar2^go, 
     c^rlight/rlight^on, c^rlight/rcar1^go, c^rlight/rcar2^go], [] ).

crash(R) :-
    failure( ag( ifthen(fromleft,all([send(rcar1^enter),send(rcar2^enter)],false) ) /\
                 ifthen(fromright,all([send(lcar1^enter),send(lcar2^enter)], false) ) ),
	    oneway, R).

deadlock(R) :-
    failure( ag( ef( allbut([tick], true) ) ), oneway, R ).

liveness(R) :-
    failure( ag( all([send(lcar1^wait)], h([send(lcar1^enter)]) ) ),
             oneway, R ).

